import sys, os
import enum

"""Just checking differences"""

class SPECIAL_MASKS(enum.Enum):

	NO_MASK = False
	BLIND_MASK = None
	DEFAULT_MASK = "*"
	ONE_REVERSE = 0xfff
	MIMI_REVERSE = 0x1ff

##GP_DEFAULT_MASK = SPECIAL_MASKS.get(os.environ.get("GETPASS_MASK" , None) , "DEFAULT_MASK")
GP_DEFAULT_MASK = getattr(SPECIAL_MASKS , os.environ.get("GETPASS_MASK" , '')  , SPECIAL_MASKS.DEFAULT_MASK  )
class OneReverse():

	def __init__(self, colored , colorset):

		if colored:
			import termcolor
			self.colored = lambda x : termcolor.colored(x , colorset[-1])

		else :
			self.colored = lambda x : x

	def mask(self):
		global password

		if len(password) == 1 :
			return self.colored(password[0])


		return self.colored(f'\b{SPECIAL_MASKS.DEFAULT_MASK.value}{password[-1]}')

	def blank(self):

		return '\b \b'

	def enter(self):
		return self.colored(f'\b{SPECIAL_MASKS.DEFAULT_MASK.value}\n')
		pass

class MimiReverse(OneReverse):

	def blank(self):

		global password

		if len(password) == 1 :
			return '\b \b'

		return self.colored(f'\b\b{password[-2]} \b')

standardize_to_str = lambda x : x if (isinstance(x , str)) else ((x.decode('utf8')) if isinstance(x , bytes) else str(x))

password = ""

class _Getch:
    """Gets a single character from standard input.  Does not echo to the
	screen."""
    def __init__(self):
        try:
            self.impl = _GetchWindows()
        except ImportError:
            self.impl = _GetchUnix()

    def __call__(self): return self.impl()

class _GetchUnix:
    def __init__(self):
        import tty, sys

    def __call__(self):
        import sys, tty, termios
        fd = sys.stdin.fileno()
        old_settings = termios.tcgetattr(fd)
        try:
            tty.setraw(sys.stdin.fileno())
            ch = sys.stdin.read(1)
        finally:
            termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
        return ch

class _GetchWindows:
    def __init__(self):
        import msvcrt

    def __call__(self):
        import msvcrt
        return msvcrt.getch()

getch = _Getch()

def getpass(	prompt : str = "Enter Password: " , mask : str = GP_DEFAULT_MASK ,
				colored : bool = False , colorset : list = None ,
				stdin = sys.stdin , stdout = sys.stderr ,
				on_error : bool = True):

	global password
	password = '' 							##Refreshing to make sure last remnants dont remain

	if stdin != sys.stdin :
		raise AttributeError(f'It is an old attribute. Now by default stdin has to be sys.stdin')

	def printex(content ):
		nonlocal stdout


	if colored:
		if colorset is None:
			colorset = ['yellow' , 'green']

	if (mask == None) or (mask == '') or (mask == b'') or (mask == SPECIAL_MASKS.BLIND_MASK):
		mask = lambda : ''
		blank = lambda  : ''
		enter = lambda : '\n'

	elif (mask == True)  or (mask == SPECIAL_MASKS.DEFAULT_MASK):

		if colored:
			mask = lambda : termcolor.colored(SPECIAL_MASKS.DEFAULT_MASK.value , colorset[-1])
			import termcolor
			blank = lambda : '\b \b'
			enter = lambda : '\n'


		else :
			mask = lambda : SPECIAL_MASKS.DEFAULT_MASK.value
			blank = lambda : '\b \b'
			enter = lambda : '\n'

	elif (mask == False ) or (mask == SPECIAL_MASKS.NO_MASK):

		if colored:

			import termcolor

			def masker(x = None):       #Param x just to avoid pre-computation of lambda
				global password
				nonlocal colorset
				return termcolor.colored(password[-1] , colorset[-1])

			mask = lambda : masker(None)
			blank = lambda : '\b \b'
			enter = lambda : '\n'

		else :

			def masker(x = None):       #Param x just to avoid pre-computation of lambda
				global password
				return password[-1]

			mask = lambda : masker(None)
			blank = lambda : '\b \b'
			enter = lambda : '\n'

	elif (mask == SPECIAL_MASKS.ONE_REVERSE):

		one_reverse = OneReverse(colored , colorset)

		mask = one_reverse.mask
		blank = one_reverse.blank
		enter = one_reverse.enter

	elif (mask == SPECIAL_MASKS.MIMI_REVERSE):

		mimi_reverse = MimiReverse(colored , colorset)

		mask = mimi_reverse.mask
		blank = mimi_reverse.blank
		enter = mimi_reverse.enter

	else :

		mask_ = str(mask)[0]

		if colored:
			import termcolor
			mask_ = termcolor.colored(mask_ , colorset[-1])
			mask = lambda : mask_

		else:
			mask = lambda : mask_

		blank = lambda : '\b \b'
		enter = lambda : '\n'

	if colored:
		import termcolor
		termcolor.cprint(prompt , color = colorset[0] , end = '' , file = stdout)
		stdout.flush()
	else:
		print(prompt , end = '' , file = stdout)
		stdout.flush()

	while True :

		try:
			character = standardize_to_str(getch.__call__())
			key = ord(character)

		except:
			continue

		if key == 13:


			stdout.write(enter())
			stdout.flush()
			return password

		elif key == 27 :
			#Escape Kye
			continue

		elif key in (8,127):

				if len(password) > 0:

					stdout.write(blank())
					stdout.flush()

					password = password[:-1]


		elif key == 3 : #Ctrl C

			if on_error:
				raise KeyboardInterrupt(f'Password Input has been interrupted by user')

			return None

		elif -1 < key < 32 :
			continue


		else :

			password += character
			stdout.write(mask())
			stdout.flush()




password_to_bytes = lambda password : (password.encode('utf8')) if isinstance(password , str) else password
password_to_str = lambda password: (password.decode('utf8')) if instance(password , bytes) else password

class PasswordError(ValueError):
	pass

class CorruptionError(ValueError):
	pass

def key_decryptor(nonce : bytes , password : bytes , rounds : int = 6):


	from hashlib import sha512 , sha256

	r = nonce
	password = password_to_bytes(password)

	for i in range(rounds):
		r = sha256(r + password).digest()

	return r

def first_block_blender(nonce : bytes , enc_password : bytes , user_password : bytes ,  rounds : int = 6):

	from hashlib import sha512

	s = nonce
	major_pass = password_to_bytes(enc_password) + password_to_bytes(user_password)


	for i in range(rounds):
		s = sha512(s + major_pass).digest()

	return s

def key_maker( password : bytes = b'',nonce : bytes = None , rounds : int = 6 , redundant : bool = True):

	if nonce == None:
		import os
		nonce = os.urandom(8)

	password = password_to_bytes(password)

	r = key_decryptor(nonce , password )

	if redundant:
		s = first_block_blender(nonce , r , password )
		return (nonce , r , s)

	return (nonce , r)



#!/usr/bin/env python3

import os, sys
PROG_PATH = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.dirname(PROG_PATH))

from .. import jihyocrypt as jihyo
from argparse import ArgumentParser, Namespace


ALL_ENC_OPS = {	'f2f' : "Encrypt Input File to an Output File" ,
				'f2b' : "Encrypt Input File to output binary stream on the stdout stream"}

ALL_DEC_OPS = {	'f2f' : "Decrypt Input File to an Output File" ,
 				'f2b' : "Decrypt Input File to a binary stream on the stdout stream",
				'f2s' : "Decrypt Input File Data to a standard python serializable object(.pickle object)"}


def bool_from_int(val):

	val  = int(val)

	if val not in (0,1):
		raise ValueError(f'Value should be 0/1')

	if val == 0:
		return False

	return True

def arguments(args : list = None ):

	parser = ArgumentParser(prog = 'jihyo' ,
	description = 'JihyoCrypt Command Line Tool. Uses Salsa20 backend with JPIC[Jihyocrypt Password Integrity Check]' ,
	epilog = f'Program stored @ "{os.path.abspath(os.path.dirname(__file__))}"')

	subparsers = parser.add_subparsers(dest = 'action')

	enc_parser = subparsers.add_parser('enc' )
	dec_parser = subparsers.add_parser('dec')

	enc_parser.add_argument(dest = 'file' , metavar = '<FILE>' , type = os.path.abspath ,
	 						help = 'File to encrypt using JihyoCrypt')

	dec_parser.add_argument(dest = 'file' , metavar = '<FILE>', type = os.path.abspath ,
							help = 'File to decrypt using JihyoCrypt')

	dec_options = dec_parser.add_argument_group(title = 'decryption mode' ,
	 											description = "Decryption Mode to be used by JihyoCrypt. Defaults to `f2f`")
	for option, htext in ALL_DEC_OPS.items():

		dec_options.add_argument(f'--{option}' ,
								action = 'store_const' , dest = 'mode' ,
								const = option , default = 'f2f' , help  = htext )

	enc_options = enc_parser.add_argument_group(title = 'encryption mode' ,
												description = "Encryption Mode to be used by JihyoCrypt. Defaults to `f2f`")

	for option, htext in ALL_ENC_OPS.items():

		enc_options.add_argument(f'--{option}' ,
								action = 'store_const' , const = option ,
								dest = 'mode' ,  default = 'f2f' , help = htext)

	for spec_parser in [enc_parser , dec_parser]:


		spec_parser.add_argument('-o' , '--output' , default = None , type = os.path.abspath , metavar = '<OUTPUT>' ,
								dest = 'output_dest' , help = 'Output file/stream')

		spec_parser.add_argument('-r' , '--redundant' , type = bool_from_int  , default = True , metavar = '<r>' ,
									dest = 'red'  , help = 'Redundancy required or not. Use 1 for true, 0 for false. TRUE by default')

		spec_parser.add_argument('-P' , '--progress_bar' , '--progress' , '--show-progress' ,
									dest = 'p_bar' , action = 'store_true' ,
									help = 'Shows progress of enc/dec. OFF by default')

		spec_parser.add_argument('-p' , '--password' ,
									dest = 'password' , action = 'store' , default = None,
									help = 'Password to encrypt/decrypt. Note: Masked input will be taken if not provided')

	if args == None:
		return parser.parse_args()

	return parser.parse_args(args)

def pretty_print(obj , indent : int = 4 , stream = sys.stdout):

	import pprint
	content = pprint.PrettyPrinter(indent = indent , stream = stream)
	content.pprint(obj)
	
def obtain_password():
    
	from .getpass import getpass
	return getpass("Enter Password: " , colored = True)
 
def MainDriver(opts : Namespace):

	# jihyo.BUFFER_SIZE = opts.buffer

	if opts.password is None:
		opts.password = obtain_password 

	if opts.mode.endswith('f') and opts.output_dest is None:
		opts.output_dest = f'{opts.file}.{"encrypted" if (opts.action == "enc") else "decrypted"}'

	if opts.file is None:
		raise ValueError(f'File to {"encrypt" if (opts.action == "enc") else "decrypt"} is not provided')

	if opts.action == 'enc':

		if opts.mode == 'f2f':

			jihyo.enc_f2f(input_file = opts.file , output_file = opts.output_dest , password = opts.password , redundant = opts.red, progress_bar = opts.p_bar)
			return None 

		elif opts.mode == 'f2b':

			r = jihyo.dec_f2b(input_file = opts.file , password = opts.password, redundant = opts.red , progress_bar = opts.p_bar)

			pretty_print(r)
			return None 

		else:
      
			raise ValueError(f'Other modes of encryption need to be done via script. Cannot be done via command line')

	elif opts.action == 'dec':

		if opts.mode == 'f2f':

			jihyo.dec_f2f(input_file = opts.file , output_file = opts.output_dest , password = opts.password , redundant = opts.red , progress_bar = opts.p_bar)
			return None 

		else:
      
			if opts.output_dest is None:
				opts.output_dest = sys.stdout 
    
			else:
				opts.ouput_dest = open(opts.output_dest , 'wb')

			if opts.mode == 'f2b':
				r = jihyo.dec_f2b(input_file = opts.file , password = opts.password , redundant = opts.red , progress_bar = opts.p_bar)

			elif opts.mode == 'f2s':
				r = jihyo.dec_f2s(input_file = opts.file , password = opts.password , redundant = opts.red , progress_bar = opts.p_bar)

			pretty_print(r , stream = opts.output_dest)
			opts.output_dest.close()

	else:
     
		raise ValueError(f'Action can only be enc/dec. Not `{opts.action}`')

if __name__ == '__main__':

	import pretty_traceback
	pretty_traceback.install()

	import colorama
	colorama.init(autoreset=True)

	opts = arguments()
	MainDriver(opts)

	colorama.deinit()



from setuptools import setup
import os 

os.chdir(os.path.dirname(os.path.abspath(__file__)))

setup(
    
    name="jihyocrypt",
    version='2.5.0.3',
    url=  'https://gitlab.com/loggerheads-with-binary/jihyocrypt/',
    download_url='http://pypi.python.org/pypi/jihyocrypt' , 
    description="A simple implementation of Salsa20 along with a nonce+password based key and hash determination function. \
    Multiple exposed functions to ease encryption and decryption to a single key based program. ",
    author="Anna Aniruddh Radhakrishnan",
    author_email = 'dev@aniruddh.ml' , 
    packages=['jihyocrypt'],
    package_dir={
        'jihyocrypt': '.',
    },
    
    install_requires = ['pycryptodome' ],
    
    include_package_data=True,
    package_data = {'jihyocrypt' : ['./README.md']} , 
    exclude_package_data={ 'jihyocrypt' : ['__main__.py' , '__pycache__' , 'getpass.py' , './setup.py' ] } ,  
    project_urls={
        'Documentation': 'https://gitlab.com/loggerheads-with-binary/jihyocrypt/README.md',
        'Source': 'https://gitlab.com/loggerheads-with-binary/jihyocrypt/',
        'Tracker': 'https://gitlab.com/loggerheads-with-binary/jihyocrypt/issues',
    },
    
    classifiers=[   
            'Operating System :: OS Independent' ,
            "Environment :: Console" , 
            'Intended Audience :: Developers',
            "Intended Audience :: System Administrators" , 
            'License :: OSI Approved :: MIT License',
            'Programming Language :: Python :: 3',
            "Natural Language :: English" ,  
            'Topic :: Security :: Cryptography' , 
    ], 
 
    long_description_content_type='text/markdown',
    long_description=open('README.md').read()
)